// DebugWin.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "DebugWin.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
IMPLEMENT_DYNCREATE(CDebugWin, CWnd)

/////////////////////////////////////////////////////////////////////////////
// CDebugWin
static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
};

CDebugWin::CDebugWin()
{
	memset(&font,0,sizeof(font));
	font.lfHeight=11; 
	font.lfCharSet=DEFAULT_CHARSET;
	font.lfOutPrecision=OUT_DEFAULT_PRECIS;
	font.lfClipPrecision=CLIP_DEFAULT_PRECIS;
	font.lfPitchAndFamily=FF_SWISS;
	strcpy(font.lfFaceName,"Fixedsys");
	Cols=CRow=CCol=0;
	FontHeight=120;
	m_CaretCreated=false;
	BlueRow=RedRow=-1;
	TopOffset=FirstRow=0;
	Data=NULL;
	TabSize=8;
	SelectionActive=DragActive=IsResized=AllowGrow=IsScrolling=UseCaret=false;
	HighlightColor=RGB(255,0,0);
	CharHeight=16; // make non-0
}

CDebugWin::~CDebugWin()
{
	if (m_WinPtr)
		*m_WinPtr=NULL;
	if (Data!=NULL)
		delete Data;	// free buffer if allocated
}


// MDI frame/child windows are deleted when they are closed, this allows the
// main app to know the window class has been deleted so it can create a new
void CDebugWin::SetWinPtr(CDebugWin ** ptr)
{
	m_WinPtr=(CDebugWin**)ptr;
}

BEGIN_MESSAGE_MAP(CDebugWin, CWnd)
	//{{AFX_MSG_MAP(CDebugWin)
	ON_WM_PAINT()
	ON_WM_ACTIVATE()
	ON_COMMAND(ID_FILE_PRINT, OnFilePrint)
	ON_WM_KILLFOCUS()
	ON_WM_SETFOCUS()
	ON_WM_VSCROLL()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_COMMAND(ID_EDIT_COPYALL, OnEditCopyAll)
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_EDIT_COPY_FILE, OnEditCopyFile)
	ON_COMMAND(ID_EDIT_COPY2FILE, OnEditCopy2File)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY2FILE, OnUpdateEditCopy2File)
	ON_WM_CHAR()
	ON_WM_CONTEXTMENU()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void CDebugWin::SetSize(int row, int col)
{
	MinCols=Cols = col;
	MinRows=Lines= Rows = row;	// set size of buff
	if (Data!=NULL)
		delete Data;
	Data = new char[(row+1)*col];	// allocate buff (a little bigger just in case)
	memset(Data,0,row*col);
	LastLine=Data+((Rows-1)*Cols);	// calc once ans use later
}

void CDebugWin::SetLine(int row,const char * s)
{
	int i=0;
	if (row>=Rows)
	{
		TRACE("Write to Row #%d past Window: %s\n",row-Rows,s);
		return;
	}
	char * d=Data+(Cols*row);
	while ((*s!='\0')&&(i<Cols))
	{
		if (*s=='\t')
		{	*d++=' ';
			i++;
			while (i%TabSize)
			{	*d++=' ';
				i++;
			}
		}
		else
		{	*d++=*s;
			i++;
		}
		s++;
	}
	if (i<Cols)
		*d='\0';	// end of line
	
}

void CDebugWin::SetChar(int row,int col,int c)
{
	int curlen;
	if ((row>=Rows)||(col>Cols))
	{
		TRACE("Write to Row past Window: %c",c);
		return;
	}
	char * d=Data+(row*Cols);	// start of row
	curlen=strlen(d);
	if (curlen<col)
	{
		memset(d+curlen,' ',col-curlen);	// fill with spaces
		if (col<Cols)
			d[col+1]='\0';
	}
	d[col]=c;
}

BOOL CDebugWin::SetText(int row, int col,const char * s,int len)
{
	char * d;
	if (len==0)
		len=strlen(s);
	if (Cols==0)
		return FALSE;	// no data so we cant do this
	if (row>=Rows)
	{
		TRACE("Write to Row past Window: %s",s);
		return FALSE;
	}
	int curlen;
	d=Data+(row*Cols);	// start of row
	curlen=strlen(d);
	if (curlen<col)
		memset(d+curlen,' ',col-curlen);	// fill with spaces
	if ((col+len)>Cols)	// past end of line
		len=Cols-col;	// truncate line
	d+=col;		// offset to col
	while ((col<Cols)&&len)
	{
		if (*s=='\t')
			while (++col%TabSize)
				*d++=' ';
		else
			*d++=*s;
		s++;
		len--;
	}
	if ((col<Cols)&&(col>curlen))
		*d='\0';	// end of line
	return TRUE;
}

void CDebugWin::AddLine(char * s)
{
	int i;
	memmove(Data,Data+Cols,(Rows-1)*Cols);	// scroll up
	memset(LastLine,0,Cols);	// clear last line
	i=strlen(s);	// number of chars
	for (int j=i-1;j>0;j--)
	{
		if (!isspace(s[j]))
			j=0;
		else
			s[j]='\0';
	}
	if (i>=Cols)
		memcpy(LastLine,s,Cols);	// put in new line
	else
		strcpy(LastLine,s);	// copy short line
	Invalidate(TRUE);	// update display
}

void CDebugWin::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	OnDraw(&dc);
}

void CDebugWin::OnDraw(CDC * dc) 
{
	CRect rect;
	RECT hrect;
//	CPaintDC dc(this); // device context for painting
	// Do not call CWnd::OnPaint() for painting messages
	CFont * OldFont,newfont;
	if (dc->IsPrinting())
	{
		dc->GetClipBox(&rect);
		OldFont=SelectFont(dc,false,newfont);
	}
	else
	{
		GetClientRect(&rect);
//		CalcWindowRect(&rect);
		OldFont=SelectFont(dc,true,newfont);
	}
	rect.top+=TopOffset;
	dc->FillSolidRect(&rect,RGB(255,255,255));
	dc->SetBkMode( TRANSPARENT ); // dont write over background color
	Lines=rect.Height()/CharHeight;	// number of displayable lines
	if (!m_CaretCreated)
	{
		CreateSolidCaret(CharWidth,CharHeight);
		UpdateCursor();
		m_CaretCreated=true;
	}
	 if (Lines>=Rows)
	{
		Lines=Rows;
		 ShowScrollBar( SB_VERT, IsScrolling=false);
	}
	else
	{
		ShowScrollBar( SB_VERT, IsScrolling=true);
		SetScrollRange(SB_VERT,0,Rows-Lines,false);
	}
	if ((FirstRow+Lines)>Rows)
		FirstRow=Rows-Lines;
	for (int i=FirstRow;i<(Lines+FirstRow);i++)
	{	int len;

		if (i==BlueRow)
		{	hrect=*LPCRECT(rect); // copy win size
			hrect.top=CharHeight*(i-FirstRow)+TopOffset;
			hrect.bottom=CharHeight*(i-FirstRow+1)+TopOffset;
			dc->FillSolidRect(&hrect,RGB(192,192,255));
		}
		else
		if (i==RedRow)
		{	hrect=*LPCRECT(rect); // copy win size
			hrect.top=CharHeight*(i-FirstRow)+TopOffset;
			hrect.bottom=CharHeight*(i-FirstRow+1)+TopOffset;
			dc->FillSolidRect(&hrect,RGB(255,192,192));
		}
		dc->SetBkColor(RGB(255,255,255));
		len=strlen(Data+(i*Cols));
		if (len>Cols)
			len=Cols;	// make sure only one line
		for (int j=0;j<len;j++)
		{	int c;
	
			c=Data[i*Cols+j];
			if (c&0x80)
				dc->SetTextColor(HighlightColor);
			else
				dc->SetTextColor(RGB(0,0,0));
			c&=0x7f;
			dc->TextOut(CharWidth*j,CharHeight*(i-FirstRow)+TopOffset,(const char *)&c,1);
		}
	}
	dc->SetTextColor(RGB(0,0,0));
	POINT cp;
	cp.y=CharHeight*(CRow-(Rows-Lines))+TopOffset;
	cp.x=CharWidth*CCol;
	if (UseCaret)
	{
		SetCaretPos(cp);
		ShowCaret();
	}
	dc->SelectObject(OldFont);
	if (SelectionActive)
		DrawRectangle(dc,TopLeft,BotRight);
}

void CDebugWin::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	CWnd::OnActivate(nState, pWndOther, bMinimized);
	Invalidate(TRUE);
}

bool CDebugWin::Show(bool show)
{
	if (show==false)
	{
		// destroy window
		ShowWindow(FALSE);
		m_IsActive=false;
	}
	else
	{
		ShowWindow(TRUE);
		SetFocus();
		m_IsActive=true;
	}
	return true; // this always succeeds?
}


bool CDebugWin::Create(CWnd * parent)
{
	CString Class;
	Class =AfxRegisterWndClass( CS_HREDRAW|CS_VREDRAW|CS_OWNDC,LoadCursor(NULL,IDC_ARROW));
//	RECT rect={10,32,300,200};
	if (CWnd::Create(Class,m_Title,
		WS_CHILD|WS_MAXIMIZEBOX|WS_MINIMIZEBOX,
		CFrameWnd::rectDefault ,parent,0)==0)
		return false;
	ModifyStyleEx(0,WS_EX_MDICHILD);
	CRow=Rows-1;	// cursor at last line
//	Show(true);
	Resize();
//	SetActiveWindow();
	return true;
}

bool CDebugWin::IsActive()
{
	return m_IsActive;
}


bool CDebugWin::GetFocus()
{
	return (CWnd::GetFocus()==(CWnd *) this) ;
}

void CDebugWin::OnFilePrint() 
{
	CPrintInfo printInfo;
	ASSERT(printInfo.m_pPD != NULL);    // must be set
	if (DoPreparePrinting(&printInfo))
	{
		printInfo.m_pPD->GetDevMode()->dmOrientation=DMORIENT_LANDSCAPE;

		// hDC must be set (did you remember to call DoPreparePrinting?)
		ASSERT(printInfo.m_pPD->m_pd.hDC != NULL);

		// gather file to print to if print-to-file selected
		CString strOutput;
		if (printInfo.m_pPD->m_pd.Flags & PD_PRINTTOFILE)
		{
			// construct CFileDialog for browsing
			CString strDef(MAKEINTRESOURCE(AFX_IDS_PRINTDEFAULTEXT));
			CString strPrintDef(MAKEINTRESOURCE(AFX_IDS_PRINTDEFAULT));
			CString strFilter(MAKEINTRESOURCE(AFX_IDS_PRINTFILTER));
			CString strCaption(MAKEINTRESOURCE(AFX_IDS_PRINTCAPTION));
			CFileDialog dlg(FALSE, strDef, strPrintDef,
				OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT, strFilter);
			dlg.m_ofn.lpstrTitle = strCaption;

			if (dlg.DoModal() != IDOK)
				return;

			// set output device to resulting path name
			strOutput = dlg.GetPathName();
		}

		// set up document info and start the document printing process
		CString strTitle;
		
		GetParentFrame()->GetWindowText(strTitle);
		if (strTitle.GetLength() > 31)
			strTitle.ReleaseBuffer(31);
		DOCINFO docInfo;
		memset(&docInfo, 0, sizeof(DOCINFO));
		docInfo.cbSize = sizeof(DOCINFO);
		docInfo.lpszDocName = strTitle;
		CString strPortName;
		int nFormatID;
		if (strOutput.IsEmpty())
		{
			docInfo.lpszOutput = NULL;
			strPortName = printInfo.m_pPD->GetPortName();
			nFormatID = AFX_IDS_PRINTONPORT;
		}

		// setup the printing DC
		CDC dcPrint;
		dcPrint.Attach(printInfo.m_pPD->m_pd.hDC);  // attach printer dc
		dcPrint.m_bPrinting = TRUE;
		OnBeginPrinting(&dcPrint, &printInfo);

		// disable main window while printing & init printing status dialog
		AfxGetMainWnd()->EnableWindow(FALSE);

		// start document printing process
		if (dcPrint.StartDoc(&docInfo) == SP_ERROR)
		{
			// enable main window before proceeding
			AfxGetMainWnd()->EnableWindow(TRUE);

			// cleanup and show error message
//			OnEndPrinting(&dcPrint, &printInfo);
//			dlgPrintStatus.DestroyWindow();
			dcPrint.Detach();   // will be cleaned up by CPrintInfo destructor
			AfxMessageBox(AFX_IDP_FAILED_TO_START_PRINT);
			return;
		}

		// Guarantee values are in the valid range
		UINT nEndPage = printInfo.GetToPage();
		UINT nStartPage = printInfo.GetFromPage();

		if (nEndPage < printInfo.GetMinPage())
			nEndPage = printInfo.GetMinPage();
		if (nEndPage > printInfo.GetMaxPage())
			nEndPage = printInfo.GetMaxPage();

		if (nStartPage < printInfo.GetMinPage())
			nStartPage = printInfo.GetMinPage();
		if (nStartPage > printInfo.GetMaxPage())
			nStartPage = printInfo.GetMaxPage();

		int nStep = (nEndPage >= nStartPage) ? 1 : -1;
		nEndPage = (nEndPage == 0xffff) ? 0xffff : nEndPage + nStep;


		// begin page printing loop
		BOOL bError = FALSE;
		for (printInfo.m_nCurPage = nStartPage;
			printInfo.m_nCurPage != nEndPage; printInfo.m_nCurPage += nStep)
		{
//			OnPrepareDC(&dcPrint, &printInfo);

			// check for end of print
			if (!printInfo.m_bContinuePrinting)
				break;

			// write current page

			// set up drawing rect to entire page (in logical coordinates)
			printInfo.m_rectDraw.SetRect(0, 0,
				dcPrint.GetDeviceCaps(HORZRES),
				dcPrint.GetDeviceCaps(VERTRES));
			dcPrint.DPtoLP(&printInfo.m_rectDraw);

			// attempt to start the current page
			if (dcPrint.StartPage() < 0)
			{
				bError = TRUE;
				break;
			}

			// must call OnPrepareDC on newer versions of Windows because
			// StartPage now resets the device attributes.
//			if (afxData.bMarked4)
//				OnPrepareDC(&dcPrint, &printInfo);

			ASSERT(printInfo.m_bContinuePrinting);

			// page successfully started, so now render the page
			OnPrint(&dcPrint, &printInfo);
			if (dcPrint.EndPage() < 0 )
			{
				bError = TRUE;
				break;
			}
		}

		// cleanup document printing process
		if (!bError)
			dcPrint.EndDoc();
		else
			dcPrint.AbortDoc();

		AfxGetMainWnd()->EnableWindow();    // enable main window

//		OnEndPrinting(&dcPrint, &printInfo);    // clean up after printing

		dcPrint.Detach();   // will be cleaned up by CPrintInfo destructor

	}
		
}

BOOL CDebugWin::DoPreparePrinting(CPrintInfo* pInfo)
{
	ASSERT(pInfo != NULL);
	ASSERT(pInfo->m_pPD != NULL);

	if (pInfo->m_pPD->m_pd.nMinPage > pInfo->m_pPD->m_pd.nMaxPage)
		pInfo->m_pPD->m_pd.nMaxPage = pInfo->m_pPD->m_pd.nMinPage;

	// don't prompt the user if we're doing print preview, printing directly,
	// or printing via IPrint and have been instructed not to ask

	CWinApp* pApp = AfxGetApp();
	if (pInfo->m_bPreview || pInfo->m_bDirect )
	{
		if (pInfo->m_pPD->m_pd.hDC == NULL)
		{
			// if no printer set then, get default printer DC and create DC without calling
			//   print dialog.
			if (!pApp->GetPrinterDeviceDefaults(&pInfo->m_pPD->m_pd))
			{
				// bring up dialog to alert the user they need to install a printer.
				if (pApp->DoPrintDialog(pInfo->m_pPD) != IDOK)
						return FALSE;
			}

			if (pInfo->m_pPD->m_pd.hDC == NULL)
			{
				// call CreatePrinterDC if DC was not created by above
				if (pInfo->m_pPD->CreatePrinterDC() == NULL)
					return FALSE;
			}
		}

		// set up From and To page range from Min and Max
 
		pInfo->m_pPD->m_pd.nFromPage = (WORD)pInfo->GetMinPage();
		pInfo->m_pPD->m_pd.nToPage = (WORD)pInfo->GetMaxPage();
	}
	else
	{
		// otherwise, bring up the print dialog and allow user to change things
		// preset From-To range same as Min-Max range
 		pInfo->m_pPD->m_pd.nFromPage = (WORD)pInfo->GetMinPage();
		pInfo->m_pPD->m_pd.nToPage = (WORD)pInfo->GetMaxPage();

		if (pApp->DoPrintDialog(pInfo->m_pPD) != IDOK)
			return FALSE;       // do not print
	}

	ASSERT(pInfo->m_pPD != NULL);
	ASSERT(pInfo->m_pPD->m_pd.hDC != NULL);
	if (pInfo->m_pPD->m_pd.hDC == NULL)
		return FALSE;

	pInfo->m_nNumPreviewPages = pApp->m_nNumPreviewPages;
	VERIFY(pInfo->m_strPageDesc.LoadString(AFX_IDS_PREVIEWPAGEDESC));
	return TRUE;
}


void CDebugWin::OnPrint(CDC * pDC, CPrintInfo * pInfo)
{
	ASSERT_VALID(pDC);
//	LPDEVMODE lpPr = (LPDEVMODE)(((CPrintDialog *)(pInfo->m_pPD))->GetDevMode());
//    lpPr->dmOrientation = DMORIENT_LANDSCAPE;

	// Override and set printing variables based on page number
	OnDraw(pDC);                    // Call Draw
}

void CDebugWin::OnBeginPrinting(CDC * dcPrint, CPrintInfo * printInfo)
{
	printInfo->SetMinPage(1);
	printInfo->SetMaxPage(1);
}

void CDebugWin::SetFontHeight(int size)
{
	FontHeight=size;
}

void CDebugWin::GotoRC(int row, int col)
{
	CRow=row;
	CCol=col;
}

void CDebugWin::Cls()
{	int i;
	for (i=0;i<Rows;i++)
		SetLine(i,"");
	GotoRC(0,0);
}

void CDebugWin::ClrEOL()
{
	// we need to implement this
	SetChar(CRow,CCol,'\0'); // terminate line
}

void CDebugWin::OnKillFocus(CWnd* pNewWnd) 
{
	CWnd::OnKillFocus(pNewWnd);
	
	// TODO: Add your message handler code here
	
}

void CDebugWin::OnSetFocus(CWnd* pOldWnd) 
{
	CWnd::OnSetFocus(pOldWnd);
	if (UseCaret)
	{
		CreateSolidCaret(0,CharHeight);
		ShowCaret();
	}
	
	// TODO: Add your message handler code here
	
}


void CDebugWin::Resize()
{
	CDC* dc=GetDC();
	CFont font,*oldfont;
	oldfont=SelectFont(dc,true,font);	// select font to calc text size
	RECT WSize,CSize;
	GetWindowRect(&WSize);
	GetClientRect(&CSize);
	// I don't know where the 2 comes from
	int xoverhead=WSize.right-WSize.left-(CSize.right-CSize.left);
	int yoverhead=2+WSize.bottom-WSize.top-(CSize.bottom-CSize.top)+GetSystemMetrics(SM_CYMENU);
	SetWindowPos(&wndTop,0,0,CharWidth*Cols+xoverhead,
				TopOffset+yoverhead+(CharHeight*Lines),SWP_DRAWFRAME|SWP_NOMOVE|SWP_NOZORDER);
//		GetSystemMetrics(SM_CYMENU)+(FontHeight*Lines)+FontHeight+40,SWP_DRAWFRAME|SWP_NOMOVE);
	dc->SelectObject(oldfont);
//	ReleaseDC(dc);	// do we need to do this?
}

/*void CDebugWin::OnSetFont() 
{
	CSlotDlg dlg;
	dlg.m_Prompt="Font height";
	dlg.m_Title="Set font height:";
	dlg.m_Slot=FontHeight/10;
	dlg.m_MaxVal=32;
	if (dlg.DoModal()==IDOK)
	{
		FontHeight=dlg.m_Slot*10;
		Resize();
	}
}
*/
void CDebugWin::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	switch (nSBCode)
	{
	case SB_PAGEDOWN:
		if ((FirstRow+Lines)<Rows)
			FirstRow+=Lines-1;
		if ((FirstRow+Lines)>=Rows)
			FirstRow=Rows-Lines;
		Invalidate();
		break;
	case SB_LINEDOWN:
		if ((FirstRow+Lines)<Rows)
			FirstRow++;
		Invalidate();
		break;
	case SB_PAGEUP:
		if (FirstRow)
			FirstRow-=Lines-1;
		if (FirstRow<0)
			FirstRow=0;
		Invalidate();
		break;
	case SB_LINEUP:
		if (FirstRow)
			FirstRow--;
		Invalidate();
		break;
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		if ((int(nPos)>=0)&&(int(nPos)<=Rows-Lines))
			FirstRow=nPos;
		Invalidate();
		break;
	};
//	if (pScrollBar!=NULL)
//	{
		SetScrollPos(SB_VERT,FirstRow);
//	}
//	CWnd::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CDebugWin::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	if (AllowGrow)
	{
		if ((CharHeight==0)||(CharWidth==0))
			return; // we must be minimized
		ChangeSize((cy-TopOffset)/CharHeight,cx/CharWidth);	// new widthd
	}
}


void CDebugWin::ChangeSize(int rows,int cols)
{
		char * old=Data;
		int oldrows=Rows;
		int oldcols=Cols;
		if (rows<MinRows)
			Rows=MinRows;
		else
			Rows=rows;
//		if (Rows<=25)
			Lines=Rows;
//		else
//			Lines=25;
		if (cols!=-1) // -1 means don't change
			if (cols<MinCols)
				Cols=MinCols;
			else
				Cols=cols;
		if (Rows<=oldrows)
			oldrows=Rows;	// copy only what is left
		else
			FirstRow=0;		// if bigger remove scroll
		Data=new char[(Rows+1)*Cols];
		memset(Data,0,(Rows+1)*Cols);
		for (int i=0;i<oldrows;i++)
			memcpy(Data+(i*Cols),old+(i*oldcols),oldcols);
		delete old;
		LastLine=Data+((Rows-1)*Cols);	// calc once ans use later
		IsResized=true;
}

void CDebugWin::OnLButtonDown(UINT nFlags, CPoint point) 
{
	TopLeft=point;
	TopLeft.y-=TopOffset;
	if (TopLeft.y>=0)
	{
		TopLeft.y-=TopLeft.y%CharHeight; // round to nearest char
		TopLeft.x-=TopLeft.x%CharWidth;
		TopLeft.y+=TopOffset;
		BotRight=TopLeft;
		DragActive=true;
		SetCapture();
		SetCursor(LoadCursor(NULL,IDC_CROSS));
		Invalidate();
	}
	//CWnd::OnLButtonDown(nFlags, point);
}

void CDebugWin::OnLButtonUp(UINT nFlags, CPoint point) 
{
	DragActive=false;
	ReleaseCapture();
//	CWnd::OnLButtonUp(nFlags, point);
}

void CDebugWin::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (DragActive)
	{
		point.y-=TopOffset;
		point.y-=point.y%CharHeight; // round to nearest char
		point.x-=point.x%CharWidth;
		point.y+=TopOffset;

		if (point!=BotRight)
		{
			SelectionActive=true;
			CDC * pDC = GetDC();
			pDC->SetROP2(R2_NOT);
			DrawRectangle(pDC,TopLeft,BotRight); // erase old
			BotRight=point;
			DrawRectangle(pDC,TopLeft,BotRight); // draw new
		}
		if (point.y>(Lines*CharHeight))
		{	
			int oldfirst=FirstRow;
			OnVScroll(SB_LINEDOWN,0,NULL);
			if (oldfirst!=FirstRow) // it did scroll
				TopLeft.y-=CharHeight;
//			OnPaint();	// redraw screen
//			Sleep(100);	// don't scroll too fast
//			point.y-=CharHeight;
//			PostMessage(WM_MOUSEMOVE,nFlags,point.x|(point.y<<16));
		}
	}
}

void CDebugWin::DrawRectangle(CDC * pDC, CPoint P0, CPoint P1)
{
	CPoint P2(P1.x,P0.y),P3(P0.x,P1.y);
	pDC->MoveTo(P0);
	pDC->LineTo(P2);
	pDC->LineTo(P1);
	pDC->LineTo(P3);
	pDC->LineTo(P0);
}

void CDebugWin::OnEditCopy() 
{
	int row1,col1,row2,col2;
	row1=FirstRow+((TopLeft.y-TopOffset)/CharHeight);
	col1=TopLeft.x/CharWidth;
	row2=FirstRow+(BotRight.y/CharHeight);
	col2=BotRight.x/CharWidth;
	CopyRange(row1,col1,row2,col2);
}

void CDebugWin::OnEditCopyAll() 
{
	CopyRange(0,0,Rows,Cols);
}

void CDebugWin::CopyRange(int row1, int col1, int row2, int col2,bool Copy2File)
{

	CString Txt;
	CFileDialog FDlg(FALSE,"txt","",OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT,"Text files |*.txt|All Files (*.*)|*.*||");
	if (Copy2File)
	{
		if (FDlg.DoModal()!=IDOK)
			return;
	}
	else
	{
		ClipData=new COleDataSource();
	}
	char * mem;
	// make sure tl to br
	if (row1>row2)
	{
		int x=row1;
		row1=row2;
		row2=x;
	}
	if (col1>col2)
	{
		int x=col1;
		col1=col2;
		col2=x;
	}
	CString Line;
	
	for (int r=row1;r<row2;r++)
	{
		Line.Empty();
		char * s=Data+(r*Cols)+col1;
		for (int c=col1;c<col2;c++)
			if (*s!='\0')
				Line+=*s++&0x7f;
			else
			{	s++;
				break;
			}
		Txt+=Line;	//add line
		if ((row2-row1)>1)	// if one line then no \n
			Txt+="\r\n";
	}
	if (Copy2File)
	{
		CFile txtfile;
		if (txtfile.Open(FDlg.GetPathName(),CFile::modeCreate|CFile::modeWrite))
		{
			txtfile.Write(Txt,Txt.GetLength());
		}
		txtfile.Close();
	}
	else
	{
		//if (Cmem==NULL)
		Cmem = ::GlobalAlloc(GMEM_MOVEABLE |GMEM_DDESHARE,Txt.GetLength()+1);
		mem=(char *)::GlobalLock(Cmem);
		memcpy(mem,Txt,Txt.GetLength()+1);
		::GlobalUnlock(Cmem);
		//ClipData->Empty();
		ClipData->CacheGlobalData(CF_TEXT,Cmem);
		ClipData->SetClipboard();
	}
	SelectionActive=false;
	Invalidate();
	
}

void CDebugWin::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(SelectionActive);
}


CFont *CDebugWin::SelectFont(CDC * dc,bool SetHeight,CFont &Cfont)
{
	CFont *oldfont;
	TEXTMETRIC tm;
//	LOGFONT font;
//	memset(&font,0,sizeof(font));
//	if (SetHeight)
		font.lfHeight=FontHeight; //rect.Height()/Rows;
//	else
//		font.lfHeight=0;	// use default
//	font.lfWeight=600;
//	font.lfCharSet=DEFAULT_CHARSET;
//	font.lfOutPrecision=OUT_DEFAULT_PRECIS;
//	font.lfClipPrecision=CLIP_DEFAULT_PRECIS;
//	font.lfPitchAndFamily=FF_SWISS;
//	strcpy(font.lfFaceName,"Courier New");
//
		if (!Cfont.CreatePointFontIndirect(&font,dc))
		AfxMessageBox("Could not create font",MB_OK);
//	if (!Cfont.CreateFont(SetHeight?FontHeight:0,0,0,0,600,0,0,0,
//		DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,
//		DEFAULT_QUALITY,FIXED_PITCH,"Ariel"))
//		AfxMessageBox("Could not create font",MB_OK);
	oldfont=dc->SelectObject(&Cfont);
	dc->GetTextMetrics(&tm);	// get font info
	CharHeight=tm.tmHeight;		// set the public size for mouse calc
//	if (SetHeight)
//		FontHeight=CharHeight;
	CharWidth=tm.tmAveCharWidth;
	// if variable pitch font, make wider
	if (tm.tmPitchAndFamily&TMPF_FIXED_PITCH)
		CharWidth+=(CharWidth*2)/5;

	return oldfont;
}

void CDebugWin::SetRed(CString & s)
{
	for (int i=0;i<s.GetLength();i++)
		if (!isspace(s.GetAt(i)))
			s.SetAt(i,s.GetAt(i)|0x80);
}

void CDebugWin::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch (nChar)
	{
	case VK_NEXT:
		OnVScroll(SB_PAGEDOWN,0,NULL);
		break;
	case VK_PRIOR:
		OnVScroll(SB_PAGEUP,0,NULL);
		break;
	case VK_UP:
		OnVScroll(SB_LINEUP,0,NULL);
		break;
	case VK_DOWN:
		OnVScroll(SB_LINEDOWN,0,NULL);
		break;
	default:
	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
	}
}

void CDebugWin::OnEditCopyFile() 
{
 	CopyRange(0,0,Rows,Cols,true);
}

void CDebugWin::OnEditCopy2File() 
{
	int row1,col1,row2,col2;
	row1=FirstRow+((TopLeft.y-TopOffset)/CharHeight);
	col1=TopLeft.x/CharWidth;
	row2=FirstRow+(BotRight.y/CharHeight);
	col2=BotRight.x/CharWidth;
	CopyRange(row1,col1,row2,col2,true);
}

void CDebugWin::OnUpdateEditCopy2File(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(SelectionActive);
}




void CDebugWin::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	m_Keys+=char(nChar);
	//CWnd::OnChar(nChar, nRepCnt, nFlags);
}


void CDebugWin::OnClose() 
{
	if (m_WinPtr)
		*m_WinPtr=NULL;
	CWnd::OnClose();
}

void CDebugWin::UpdateCursor()
{
	CPoint point;
	point.x=CCol*CharWidth;
	point.y=CRow*CharHeight;
	SetCaretPos(point);
	ShowCaret();
}
