// Comm.cpp : implementation file
//

#include "stdafx.h"
#include "comm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//extern CPBXInitApp theApp;

//#define USE_OVERLAPIO
/////////////////////////////////////////////////////////////////////////////
// CComm

CComm::CComm() :  hComm(NULL)
{

}

CComm::~CComm()
{
	CloseComm();
}

bool CComm::OpenComm(int Port,int baud,bool flow)
{	bool flag=false;
	CString s;
	m_FlowOn=flow;
	s.Format("COM%d",Port);	// make comx string
	if ((hComm=CreateFile(s,GENERIC_READ | GENERIC_WRITE,
			0,
			NULL,	// no security
			OPEN_EXISTING,	// there must be a port
			FILE_ATTRIBUTE_NORMAL,	// async i/o
			NULL)) == (HANDLE) -1)
		return false;
	else
	{
		SetCommMask(hComm, EV_RXCHAR|EV_TXEMPTY);	// receive events
		GetCommState(hComm, &DCB);	// read current config

		Baud= baud;	// default

		ReInitPort();
		flag = true;
	}
	return flag;
}



void CComm::CloseComm()
{
	if (hComm)
		SetCommMask(hComm,0);	// turn off mask should signal wait to end
	if (hComm)
		CloseHandle(hComm);
	hComm=NULL;
}


//
// control DTR (used to reset board)
//
void CComm::SetDTR(bool flag)
{
	if (flag)
		EscapeCommFunction(hComm,SETDTR);
	else
		EscapeCommFunction(hComm,CLRDTR);
}

void CComm::SetBaud(int baud)
{
		Baud=baud;
		ReInitPort();
//		ReInit=TRUE;	// tell RX task to change baud rate
//		SetCommMask(hComm,0);	// will cause rx task to run
//		while (ReInit)
//			Sleep(10);	// waif for RX task to change baud
}


int CComm::SendBlock(void * buff,int len)
{	DWORD length;
	int bytes=0;
	char *s=(char *)buff;
/*#ifdef USE_OVERLAPIO
	OVERLAPPED os;
	memset(&os,0,sizeof(OVERLAPPED));	// set 0
	if (WriteFile(hComm,buff,len,&length,&os)!=TRUE)
		TRACE("SendBlock Write err\n");
	if ((length=GetOverlappedResult(hComm,&os,&length,TRUE))!=0)
		TRACE("SendBlock GetOverlappedResult returned %x\n",length);

#else*/
	int retries=10;
	while (bytes!=len)
	{
		WriteFile(hComm,s,len-bytes,&length,NULL);
		bytes+=length;
		s+=length;
		if (bytes!=len)
		{
			if (--retries==0)
				return bytes;
			Sleep(100);
		}
	}
//#endif
	return bytes;
}


void CComm::ReInitPort()
{
	GetCommState(hComm, &DCB);	// read current config
	DCB.BaudRate = Baud;
	DCB.ByteSize =8;
	DCB.StopBits = ONESTOPBIT;
	DCB.fParity = FALSE;
	DCB.Parity = NOPARITY;
	DCB.fOutxCtsFlow = m_FlowOn?TRUE:FALSE;	// no handshaking
	DCB.fOutxDsrFlow = m_FlowOn?TRUE:FALSE;
	DCB.fDtrControl = DTR_CONTROL_ENABLE;
	DCB.fDsrSensitivity = FALSE;
	DCB.fOutX = FALSE;	// no flow control
	DCB.fInX = FALSE;
	DCB.fNull= FALSE;
	DCB.fRtsControl = RTS_CONTROL_ENABLE; // rts follows data
	DCB.fBinary = TRUE;
	DCB.fAbortOnError=FALSE;

	SetCommState(hComm,&DCB);
	SetupComm(hComm,2048,2048);
	COMMTIMEOUTS CommTimeOuts;
	CommTimeOuts.ReadIntervalTimeout = 0;
	CommTimeOuts.ReadTotalTimeoutMultiplier = 2*CBR_19200/DCB.BaudRate ;
	CommTimeOuts.ReadTotalTimeoutConstant = 1;
	// CBR_9600 is approximately 1byte/ms. For our purposes, allow
	// double the expected time per character for a fudge factor.
	CommTimeOuts.WriteTotalTimeoutMultiplier = 2*CBR_19200/DCB.BaudRate;
	CommTimeOuts.WriteTotalTimeoutConstant = 500 ;
	SetCommTimeouts( hComm, &CommTimeOuts ) ;
	SetCommMask(hComm,EV_RXCHAR|EV_TXEMPTY);	// we wan't rx events
}

void CComm::SetBreak(bool flag)
{
	if (flag)
		SetCommBreak(hComm);
	else
		ClearCommBreak(hComm);
}

bool CComm::IsActive()
{
	return (hComm!=NULL);
}

HANDLE CComm::GetHandle()
{
	return hComm;
}

int CComm::GetBaud()
{
	return Baud;
}

bool CComm::GetLine(CString & s)
{
	int timeout=10;
	DWORD bytes;
	int c=0;
	s.Empty();
	while (timeout&&(c!='\n'))
	{	
		bytes=0;
		if (ReadFile(hComm,&c,1,&bytes,NULL)&&bytes)
		{
			if ((c!='\r')&&(c!='\n'))
				s+=(char)c;
			timeout=5;
		}
		else
			timeout--;
	}
	if ((timeout)&&s.GetLength())
		return true;	// good read
	return false;
}

bool CComm::CheckDCD()
{
	unsigned long status;
	GetCommModemStatus(hComm,&status);
	return (status&MS_RLSD_ON)!=0; 
}


// read block of data, returns # read
int CComm::GetBlock(void * buff, int len)
{
	int timeout=10;
	DWORD bytes;
	char * data=(char *)buff;
	int count=0;
	int c=0;
	while (timeout&&count<len)
	{
		ReadFile(hComm,&c,1,&bytes,NULL);
		if (bytes)
		{
			*data++=c;
			count++;
			timeout=5; // once we get data use short timeout
		}
		else
		{
			timeout--;
			Sleep(5);
		}
	}
	return count;

}

bool CComm::CheckCTS()
{
	unsigned long status;
	GetCommModemStatus(hComm,&status);
	return (status&MS_CTS_ON)!=0; 
}

bool CComm::CheckDSR()
{
	unsigned long status;
	GetCommModemStatus(hComm,&status);
	return (status&MS_DSR_ON)!=0; 
}

bool CComm::CheckRing()
{
	unsigned long status;
	GetCommModemStatus(hComm,&status);
	return (status&MS_RING_ON)!=0; 
}

void CComm::SetRTS(bool flag)
{
	if (flag)
		EscapeCommFunction(hComm,SETRTS);
	else
		EscapeCommFunction(hComm,CLRRTS);
}

int CComm::SendChar(int ch)
{
	DWORD length;
	WriteFile(hComm,&ch,1,&length,NULL);
	return length;
}

int CComm::SendString(CString str)
{
	DWORD length;
	WriteFile(hComm,LPCSTR(str),str.GetLength(),&length,NULL);
	return length;
}

void CComm::WaitTX()
{
	unsigned long mask=0;
	while ((mask&EV_TXEMPTY)==0)
		WaitCommEvent(hComm,&mask,NULL);
}

void CComm::PurgeRX()
{
	PurgeComm(hComm,PURGE_RXCLEAR);
}
